/****************************************************************************
*                                                                           *
*   Coupling program for the evolutionary optimizer:                        *
*   Calling a completely external simulator module                          *
*   Version 0.5                                                             *
*   19.4.94            (C) Petra Roosen                                     *
*                                                                           *
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#ifdef UNIX
  #include <unistd.h>
#else
  #include <process.h>
#endif

#include "epo.h"


/**** global variables: ****/

Paraset Indi;
int     Parazahl, Runde;
Iniset  Limits[5];
FILE    *infile, *outfile, *flagfile;
char    Transdir[250];
int     sleeptime=1;
int     optidelay;

/**** function prototypes: ****/

void    main(void);
void    init(void);
void    rufe(void);

#include "eposim.c"

/**************************************************************************/
/**************************************************************************/
/******************* Begin of target function determination ***************/

void rufe()  /* Call of an external simulator */
{
  double  Ergebnis;
  char    Aufruf[250];
  FILE    *rausfile;

  /*************************************************************************
  In this example an external simulator is invoked by transferring 5 command
  line parameters. A simple program returning rms deviation of entries in a
  data file from a theoretical gaussian curve was chosen as an "external
  simulator" example.

  The "GAUSSFIT.EXE" program expects the following parameters:
    1: measurement data file (consisting of {x,y} data pairs, as given in
       the "REIN" file.
    2: Amplitude of the theoretical gaussian distribution (1st fit parameter)
    3: Average of the gaussian distribution (2nd fit parameter)
    4: standard deviation of the gaussian distribution (3rd fit parameter)
    5: Name of the result file to be written.

  Once again it is sensible to use a RAM disk for transfer file handling.
  Remember to care about the formatting of the external simulator's return
  file, as you usually cannot control it by yourself.

  --------------------------------------------------------------------------

  Hier wird beispielhaft ein externer Simulator mit 5 Uebergabeparametern
  aufgerufen, z.B. das beigelegte Gaussfit-Program, das als Parameter

    1: Messdatenfile-Name (bestehend aus {x,y}-Paaren gemaess dem File "rein"
    2: Amplitude der theoretischen Gaussverteilung (1. Fitparameter)
    3: Mittenwert der theor. Gaussvert. (2. Fitparameter)
    4: Standardabweichung der theor. Gaussvert. (3. Fitparameter)
    5: Ergebnisdatenfile-Name

  erwartet. Auch hier ist es wieder sinnvoll, Uebergabefiles auf einer
  RAM-Disk zu verwalten.

  Beim Auswerten des vom externen Simulators geschriebenen Ergebnisfiles
  muss auf dessen Formatierungen Ruecksicht genommen werden!
  *************************************************************************/

  sprintf(Aufruf, "gaussfit rein %f %f %f raus\n",
          Indi.koeff[0], Indi.koeff[1], Indi.koeff[2]);

  system(Aufruf);

  rausfile = fopen("raus", "r");
  fscanf(rausfile, "%*s %*s %*s %lf", &Ergebnis);
  fclose(rausfile);

  Indi.Ergebnis = Ergebnis;
}

/******************** End of target function determination *****************/
/***************************************************************************/
/***************************************************************************/

void main()
{
  bool klappt;     

  init();          /* for transfer directory determination only */

  forever
  {
    Runde++;
    klappt = empfange();
    rufe();
    printf("%d. simulator result: %G           sleep time %d\n",
           Runde, Indi.Ergebnis, sleeptime);
    klappt = verschicke();
  }
}


void init()
{
  FILE    *fp;
  char    dummy[250];

  printf("\n\n");
  printf("  **************************************************\n");
  printf("  * External simulator addressing module of the    *\n");
  printf("  * Evolutionary Parameter Optimizer (EPO)         *\n");
  printf("  * for general optimization problems              *\n");
  printf("  *                                                *\n");
  printf("  * Shareware version 0.5    (C)3/94  Petra Roosen *\n");
  printf("  **************************************************\n\n");


  /** Scanning of the STRATEG.DAT file to get the transfer directory name **/
#ifdef UNIX
  if ( (fp=fopen("strateg.unx","r")) )
#else
  if ( (fp=fopen("strateg.os2","r")) )
#endif
    {
      fscanf(fp, "%s", dummy);
      if ( strcmp(dummy, Version) )
      {
        printf("\007Wrong initialization file at the identification header!\n");
        sleep(5);
        exit(0);
      }
      fscanf(fp, "%*s");
      fscanf(fp, "%*s %*s");
      fscanf(fp, "%s %d", dummy, &optidelay);
      if ( strcmp(dummy, "optidelay") )
        {
          printf("\007Faulty initialization at \"optidelay\"!\n");
          sleep(5);
          exit(0);
        }
      fscanf(fp, "%*s %*d");
      fscanf(fp, "%*s %*d");
      fscanf(fp, "%*s %*d");
      fscanf(fp, "%*s %*d");
      fscanf(fp, "%*s %*s");
      fscanf(fp, "%*s %*s");
      fscanf(fp, "%s %s", dummy, Transdir);
      if ( strcmp(dummy, "transfer_dir") )
      {
        printf("\007Faulty initialization file at \"transfer_dir\"!\n");
        sleep(5);
        exit(0);
      }
      fclose(fp);
    }
}


